@@ -74,7 +74,8 @@ function executeScriptHelper(script, statements, options, locals) {
         // Jump?
         } else if (statementKey === 'jump') {
             // Evaluate the expression (if any)
-            if (!('expr' in statement.jump) || evaluateExpression(statement.jump.expr, options, locals, false, script, statement)) {
+            if (!('expr' in statement.jump) ||
+                valueBoolean(evaluateExpression(statement.jump.expr, options, locals, false, script, statement))) {
                 // Find the label
                 if (labelIndexes !== null && statement.jump.label in labelIndexes) {
                     ixStatement = labelIndexes[statement.jump.label];
@@ -222,7 +223,7 @@ export function evaluateExpression(expr, options = null, locals = null, builtins
         if (funcName === 'if') {
             const [valueExpr = null, trueExpr = null, falseExpr = null] = expr.function.args ?? [];
             const value = (valueExpr !== null ? evaluateExpression(valueExpr, options, locals, builtins, script, statement) : false);
-            const resultExpr = (value ? trueExpr : falseExpr);
+            const resultExpr = (valueBoolean(value) ? trueExpr : falseExpr);
             return resultExpr !== null ? evaluateExpression(resultExpr, options, locals, builtins, script, statement) : null;
         }
 
