@@ -80,7 +80,7 @@ async function executeScriptHelperAsync(script, statements, options, locals) {
         } else if (statementKey === 'jump') {
             // Evaluate the expression (if any)
             if (!('expr' in statement.jump) ||
-                await evaluateExpressionAsync(statement.jump.expr, options, locals, false, script, statement)) {
+                valueBoolean(await evaluateExpressionAsync(statement.jump.expr, options, locals, false, script, statement))) {
                 // Find the label
                 if (labelIndexes !== null && statement.jump.label in labelIndexes) {
                     ixStatement = labelIndexes[statement.jump.label];
@@ -289,7 +289,7 @@ export async function evaluateExpressionAsync(expr, options = null, locals = nul
         if (funcName === 'if') {
             const [valueExpr, trueExpr = null, falseExpr = null] = expr.function.args;
             const value = await evaluateExpressionAsync(valueExpr, options, locals, builtins, script, statement);
-            const resultExpr = (value ? trueExpr : falseExpr);
+            const resultExpr = (valueBoolean(value) ? trueExpr : falseExpr);
             return resultExpr !== null ? evaluateExpressionAsync(resultExpr, options, locals, builtins, script, statement) : null;
         }
 
