@@ -7,7 +7,6 @@ import {lineChartElements, validateLineChart} from './lineChart.js';
 import {valueArgsModel, valueArgsValidate} from '../bare-script/lib/value.js';
 import {encodeQueryString} from '../schema-markdown/lib/encode.js';
 import {markdownHeaderId as markdownModelHeaderId} from '../markdown-model/lib/elements.js';
-import {parseSchemaMarkdown} from '../schema-markdown/lib/parser.js';
 import {schemaMarkdownDoc} from '../schema-markdown-doc/lib/schemaMarkdownDoc.js';
 import {validateElements} from '../element-model/lib/elementModel.js';
 
@@ -141,8 +140,17 @@ const documentSetFocusArgs = valueArgsModel([
 // $doc:
 // $doc: myAppMain()
 // $doc: ```
-// $arg callback: The keydown event callback function that takes a single
-// $arg callback: [key event object](model.html#var.vName='DocumentKeyEvent') argument, `event`.
+// $arg callback: The keydown event callback function, which takes a single `event` object that has
+// $arg callback: the following attributes:
+// $arg callback: - `key` - The key value (e.g., "a", "Enter", "ArrowUp")
+// $arg callback: - `code` - The physical key code (e.g., "KeyA", "Enter")
+// $arg callback: - `keyCode` - The legacy numeric code (e.g., 65 for 'a')
+// $arg callback: - `ctrlKey` - If true, the control key is pressed
+// $arg callback: - `altKey` - If true, the alt key is pressed
+// $arg callback: - `shiftKey` - If true, the shift key is pressed
+// $arg callback: - `metaKey` - If true, the cmd key is pressed
+// $arg callback: - `repeat` - If true, the key is held down
+// $arg callback: - `location` - 0=standard, 1=left, 2=right
 function documentSetKeyDown(args, options) {
     const [callback] = valueArgsValidate(documentSetKeyDownArgs, args);
     const {runtime} = options;
@@ -180,42 +188,6 @@ const documentSetKeyDownArgs = valueArgsModel([
 ]);
 
 
-export const documentKeyEventTypes = parseSchemaMarkdown(`\
-group "Events"
-
-
-# A document key event object
-struct DocumentKeyEvent
-
-    # The key value (e.g., "a", "Enter", "ArrowUp")
-    string key
-
-    # The physical key code (e.g., "KeyA", "Enter")
-    string code
-
-    # The legacy numeric code (e.g., 65 for 'a')
-    string keyCode
-
-    # If true, the control key is pressed
-    bool ctrlKey
-
-    # If true, the alt key is pressed
-    bool altKey
-
-    # If true, the shift key is pressed
-    bool shiftKey
-
-    # If true, the cmd key is pressed
-    bool metaKey
-
-    # If true, the key is held down
-    bool repeat
-
-    # 0=standard, 1=left, 2=right
-    int location
-`);
-
-
 // $function: documentSetReset
 // $group: Document
 // $doc: Set the document reset element
